@@ -1,12 +1,26 @@
+"""
+pyautomation for Python 3.
+Author: iamtony.ca@gmail.com
+Source: https://github.com/changgwak/python-automation
+
+This module is for Automation on Windows os.
+With the pyautomation package, you can control your GUI automatically while simultaneously controlling the mouse and keyboard physically, similar to how selenium automates web browsers.
+Read 'readme.md' for help.
+
+pyautomation is shared under the MIT Licene.
+This means that the code can be freely copied and distributed, and costs nothing to use.
+"""
+
+
 import cv2
 import numpy as np
 from .modules import mss
+# import mss
 import logging
 import asyncio
-from typing import Optional, Tuple, List, Dict, Any
-from concurrent.futures import ThreadPoolExecutor
+from typing import Optional, Tuple, List
 import aiofiles
-import yaml     #pyyaml
+import yaml  # pyyaml
 import argparse
 from pydantic import BaseModel, ValidationError, field_validator
 from injector import Injector, inject, Module, singleton, provider
@@ -15,6 +29,7 @@ from dataclasses import dataclass, field
 # Structured logging configuration
 logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')
 
+
 class ConfigModel(BaseModel):
     monitor_index: int
     ratio: float
@@ -28,33 +43,27 @@ class ConfigModel(BaseModel):
             raise ValueError('ratio must be between 0 and 1')
         return v
 
+
 class ConfigModule(Module):
     def __init__(self, config_path: str = None, **kwargs):
         self.config_path = config_path
         self.kwargs = kwargs
 
-        # print(config_path, kwargs)
-
-
     @singleton
     @provider
     def provide_config(self) -> ConfigModel:
         import os
 
-        # Default parameters
         config_dict = {
-            'monitor_index': 1, 
-            'ratio': 0.7, 
-            'min_match_count': 15, 
-            'template_path': 'myvenv/imgs/fastcampus_business.JPG',
+            'monitor_index': 0,
+            'ratio': 0.7,
+            'min_match_count': 15,
+            'template_path': 'myvenv/imgs/sports.jpg',
             'show': False
-            }
+        }
 
         if self.config_path is not None:
-            # print("# Check if input is a file path")
-            # Check if input is a file path
             if os.path.isfile(self.config_path):
-                # print("os.path.isfile(self.config_path):")
                 with open(self.config_path, 'r') as file:
                     file_data = yaml.safe_load(file)
                     config_dict.update(file_data)
@@ -67,7 +76,6 @@ class ConfigModule(Module):
                     return None
         else:
             config_dict.update(self.kwargs)
-            # print("result of config_dict: ", config_dict)
 
         try:
             return ConfigModel(**config_dict)
@@ -75,6 +83,7 @@ class ConfigModule(Module):
             logging.error(f"Configuration validation error: {e}")
             raise
 
+
 @dataclass
 class ImageMatcher:
     config: ConfigModel
@@ -84,13 +93,19 @@ class ImageMatcher:
     matches: Optional[List] = field(init=False, default=None)
     good_matches: Optional[List] = field(init=False, default=None)
     object_location: Optional[np.ndarray] = field(init=False, default=None)
-    object_center: Optional[Tuple[int, int]] = field(init=False, default=None)
+    # object_center: Optional[Tuple[int, int]] = field(init=False, default=None)
+    object_center: Optional[List[int]] = field(init=False, default=None)
+    
+    monitor_shift_left: int = field(init=False, default=0)
+    monitor_shift_top: int = field(init=False, default=0)
+
 
     @inject
     def __post_init__(self):
         pass
 
     async def load_image(self, path: str, grayscale: bool = True) -> np.ndarray:
+        """Asynchronously load image from file."""
         async with aiofiles.open(path, mode='rb') as f:
             image_data = await f.read()
         image_array = np.frombuffer(image_data, np.uint8)
@@ -101,9 +116,14 @@ class ImageMatcher:
         logging.info(f"Image loaded from path: {path}")
         return image
 
+
     async def capture_screen(self) -> None:
+        """Asynchronously capture the screen."""
         with mss.mss() as sct:
             monitor = sct.monitors[self.config.monitor_index]  # Configurable monitor index
+            # print(monitor)
+            self.monitor_shift_left = monitor['left']
+            self.monitor_shift_top = monitor['top']
             screenshot = sct.grab(monitor)
             img = np.array(screenshot)
             self.screenshot_image = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
@@ -129,23 +149,28 @@ class ImageMatcher:
 
     def find_object_location(
         self, kp1: List[cv2.KeyPoint], kp2: List[cv2.KeyPoint], good_matches: List, min_match_count: int
-        ) -> Tuple[Optional[Tuple[int, int]], Optional[np.ndarray]]:
+    ) -> Tuple[Optional[Tuple[int, int]], Optional[np.ndarray]]:
         if len(good_matches) > min_match_count:
             src_pts = np.float32([kp1[m.queryIdx].pt for m in good_matches]).reshape(-1, 1, 2)
             dst_pts = np.float32([kp2[m.trainIdx].pt for m in good_matches]).reshape(-1, 1, 2)
             M, mask = cv2.findHomography(src_pts, dst_pts, cv2.RANSAC, 5.0)
 
             h, w = self.template_image.shape[:2]
-            pts = np.float32([[0, 0], [0, h-1], [w-1, h-1], [w-1, 0]]).reshape(-1, 1, 2)
+            pts = np.float32([[0, 0], [0, h - 1], [w - 1, h - 1], [w - 1, 0]]).reshape(-1, 1, 2)
             dst = cv2.perspectiveTransform(pts, M)
 
             self.object_location = np.int32(dst)
-            self.object_center = (int((dst[0][0][0] + dst[2][0][0]) / 2), int((dst[0][0][1] + dst[2][0][1]) / 2))
+            # self.object_center = (int((dst[0][0][0] + dst[2][0][0]) / 2), int((dst[0][0][1] + dst[2][0][1]) / 2))
+            self.object_center = [int((dst[0][0][0] + dst[2][0][0]) / 2), int((dst[0][0][1] + dst[2][0][1]) / 2)]
+
+            self.object_center[0] += self.monitor_shift_left
+            self.object_center[1] += self.monitor_shift_top
+            
         else:
             logging.warning(f"Not enough matches are found - {len(good_matches)}/{min_match_count}")
             self.object_location = None
             self.object_center = None
-        
+        # print(f"len(good_matches): {len(good_matches)}")
         return self.object_center, self.object_location
 
     def draw_object_location(self, color: Tuple[int, int, int] = (0, 255, 0), thickness: int = 3) -> None:
@@ -172,37 +197,41 @@ class ImageMatcher:
     async def run(self) -> None:
         await self.process_image()
         logging.info(f"Object center: {self.object_center}")
-        
-        if self.config.show is False : pass
-        elif self.config.show is True : self.draw_object_location()
 
+        if self.config.show:
+            self.draw_object_location()
 
-    async def get_object_location(self) -> Tuple[Optional[Tuple[int, int]], Optional[np.ndarray]]:
-        await self.run()
+    def run_sync(self):
+        """Run the async method synchronously using an event loop."""
+        asyncio.run(self.run())
+
+    def get_object_location_sync(self):
+        """Run the async method synchronously and return the object location."""
+        asyncio.run(self.run())
         return self.object_center, self.object_location
 
+
 def parse_args(path) -> argparse.Namespace:
     parser = argparse.ArgumentParser(description="Image Matcher Configuration")
     parser.add_argument('--config', type=str, help='Path to the configuration file', default=path)
     return parser.parse_args()
 
+
 def main(path=None, **kwargs):
     args = parse_args(path)
-    injector = Injector([ConfigModule(config_path= args.config, **kwargs)])
+    injector = Injector([ConfigModule(config_path=args.config, **kwargs)])
     config = injector.get(ConfigModel)
     matcher = ImageMatcher(config=config)
-    asyncio.run(matcher.run())
+    matcher.run_sync()
+    return matcher
+
+def image_matcher(path=None, **kwargs):
+    args = parse_args(path)
+    injector = Injector([ConfigModule(config_path=args.config, **kwargs)])
+    config = injector.get(ConfigModel)
+    matcher = ImageMatcher(config=config)
+    matcher.run_sync()
     return matcher
 
 if __name__ == "__main__":
     main()
-
-
-
-# import pyautovision
-# # a = pyautovision.main(r'myvenv/config/autovision.yaml')
-# # a = pyautovision.main()
-# a = pyautovision.main(monitor_index=1, min_match_count=15)
-# # a = pyautovision.main(monitor_index=1, min_match_count=15, template_path="myvenv/imgs/write_email.JPG")
-
-
